﻿Imports System.Drawing.Drawing2D

Public Class Form1
    ' Draw some text along a line segment.
    Private Sub Form1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAliasGridFit
        e.Graphics.SmoothingMode = SmoothingMode.AntiAlias
        e.Graphics.InterpolationMode = InterpolationMode.High

        ' Draw some text along some line segments.
        DrawOnSegment(e.Graphics, New PointF(20, 20), New PointF(330, 120), _
                "This is some text above a line segment.", True)
        DrawOnSegment(e.Graphics, New PointF(20, 20), New PointF(330, 120), _
                "This is some text below a line segment.", False)

        DrawOnSegment(e.Graphics, New PointF(330, 200), New PointF(20, 120), _
                "This is some text above a line segment.", True)
        DrawOnSegment(e.Graphics, New PointF(330, 200), New PointF(20, 120), _
                "This is some text below a line segment.", False)
    End Sub
    ' Draw some text.
    Private Sub DrawOnSegment(ByVal gr As Graphics, ByVal start_point As PointF, ByVal end_point As PointF, ByVal txt As String, ByVal text_above_segment As Boolean)
        Dim start_ch As Integer = 0

        gr.DrawLine(Pens.Green, start_point, end_point)
        DrawTextOnSegment(gr, Brushes.Blue, Me.Font, txt, _
            start_ch, start_point, end_point, text_above_segment)
    End Sub

    ' Draw some text along a line segment.
    ' Leave char_num pointing to the next character to be drawn.
    ' Leave start_point holding the coordinates of the last point used.
    Private Sub DrawTextOnSegment(ByVal gr As Graphics, ByVal brush As Brush, ByVal font As Font, ByVal txt As String, ByRef first_ch As Integer, ByRef start_point As PointF, ByVal end_point As PointF, ByVal text_above_segment As Boolean)
        Dim dx As Single = end_point.X - start_point.X
        Dim dy As Single = end_point.Y - start_point.Y
        Dim dist As Single = CSng(Math.Sqrt(dx * dx + dy * dy))
        dx /= dist
        dy /= dist

        ' See how many characters will fit.
        Dim last_ch As Integer = first_ch
        While (last_ch < txt.Length)
            Dim test_string As String = txt.Substring(first_ch, last_ch - first_ch + 1)
            If (gr.MeasureString(test_string, Font).Width > dist) Then
                ' This is one too many characters.
                last_ch -= 1
                Exit While
            End If
            last_ch += 1
        End While
        If (last_ch < first_ch) Then Return
        If (last_ch >= txt.Length) Then last_ch = txt.Length - 1
        Dim chars_that_fit As String = txt.Substring(first_ch, last_ch - first_ch + 1)

        ' Rotate and translate to position the characters.
        Dim state As GraphicsState = gr.Save()
        If (text_above_segment) Then
            gr.TranslateTransform(0, _
                -gr.MeasureString(chars_that_fit, font).Height, _
                MatrixOrder.Append)
        End If
        Dim angle As Single = CSng(180 * Math.Atan2(dy, dx) / Math.PI)
        gr.RotateTransform(angle, MatrixOrder.Append)
        gr.TranslateTransform(start_point.X, start_point.Y, MatrixOrder.Append)

        ' Draw the characters that fit.
        gr.DrawString(chars_that_fit, font, brush, 0, 0)

        ' Restore the saved state.
        gr.Restore(state)

        ' Update first_ch and start_point.
        first_ch = last_ch + 1
        Dim text_width As Single = gr.MeasureString(chars_that_fit, font).Width
        start_point = New PointF( _
            start_point.X + dx * text_width, _
            start_point.Y + dy * text_width)
    End Sub
End Class
